@@ -74,6 +74,9 @@ class TestTranformations:
             "WRITE_DISPOSITION": "WRITE_APPEND",
             "TIME_PARTITIONING": None,
             "CLUSTER_FIELDS": ["field1", "field2"],
+            "PARTITION_EXPIRATION": None,
+            "REQUIRED_PARTITION_FILTER": False,
+            "TABLE_EXPIRATION": None,
             "SCHEMA_UPDATE_OPTIONS": ["ALLOW_FIELD_ADDITION"],
             "DESTINATION_DATA_MART": None,
             "DEPENDS_ON": [],
